@@ -1,4 +1,4 @@
-/*	$OpenBSD: main.c,v 1.266 2024/09/04 15:46:43 job Exp $ */
+/*	$OpenBSD: main.c,v 1.278 2025/01/03 10:14:32 job Exp $ */
 /*
  * Copyright (c) 2021 Claudio Jeker <claudio@openbsd.org>
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
@@ -60,13 +60,12 @@ volatile sig_atomic_t killme;
 void	suicide(int sig);
 
 static struct filepath_tree	fpt = RB_INITIALIZER(&fpt);
-static struct msgbuf		procq, rsyncq, httpq, rrdpq;
+static struct msgbuf		*procq, *rsyncq, *httpq, *rrdpq;
 static int			cachefd, outdirfd;
 
-const char	*bird_tablename = "ROAS";
-
 int	verbose;
 int	noop;
+int	excludeas0 = 1;
 int	excludeaspa;
 int	filemode;
 int	shortlistmode;
@@ -182,7 +181,7 @@ entity_write_req(const struct entity *ent)
 	io_str_buffer(b, ent->file);
 	io_str_buffer(b, ent->mftaki);
 	io_buf_buffer(b, ent->data, ent->datasz);
-	io_close_buffer(&procq, b);
+	io_close_buffer(procq, b);
 }
 
 static void
@@ -208,7 +207,7 @@ entity_write_repo(const struct repo *rp)
 	io_str_buffer(b, altpath);
 	io_buf_buffer(b, NULL, 0); /* ent->mftaki */
 	io_buf_buffer(b, NULL, 0); /* ent->data */
-	io_close_buffer(&procq, b);
+	io_close_buffer(procq, b);
 	free(path);
 	free(altpath);
 }
@@ -279,7 +278,7 @@ rrdp_file_resp(unsigned int id, int ok)
 	io_simple_buffer(b, &type, sizeof(type));
 	io_simple_buffer(b, &id, sizeof(id));
 	io_simple_buffer(b, &ok, sizeof(ok));
-	io_close_buffer(&rrdpq, b);
+	io_close_buffer(rrdpq, b);
 }
 
 void
@@ -296,7 +295,7 @@ rrdp_fetch(unsigned int id, const char *uri, const char *local,
 	io_str_buffer(b, uri);
 
 	rrdp_session_buffer(b, s);
-	io_close_buffer(&rrdpq, b);
+	io_close_buffer(rrdpq, b);
 }
 
 void
@@ -308,7 +307,7 @@ rrdp_abort(unsigned int id)
 	b = io_new_buffer();
 	io_simple_buffer(b, &type, sizeof(type));
 	io_simple_buffer(b, &id, sizeof(id));
-	io_close_buffer(&rrdpq, b);
+	io_close_buffer(rrdpq, b);
 }
 
 /*
@@ -325,7 +324,7 @@ rsync_fetch(unsigned int id, const char *uri, const char *local,
 	io_str_buffer(b, local);
 	io_str_buffer(b, base);
 	io_str_buffer(b, uri);
-	io_close_buffer(&rsyncq, b);
+	io_close_buffer(rsyncq, b);
 }
 
 void
@@ -338,7 +337,7 @@ rsync_abort(unsigned int id)
 	io_str_buffer(b, NULL);
 	io_str_buffer(b, NULL);
 	io_str_buffer(b, NULL);
-	io_close_buffer(&rsyncq, b);
+	io_close_buffer(rsyncq, b);
 }
 
 /*
@@ -355,7 +354,7 @@ http_fetch(unsigned int id, const char *uri, const char *last_mod, int fd)
 	io_str_buffer(b, last_mod);
 	/* pass file as fd */
 	ibuf_fd_set(b, fd);
-	io_close_buffer(&httpq, b);
+	io_close_buffer(httpq, b);
 }
 
 /*
@@ -376,7 +375,7 @@ rrdp_http_fetch(unsigned int id, const char *uri, const char *last_mod)
 	io_simple_buffer(b, &type, sizeof(type));
 	io_simple_buffer(b, &id, sizeof(id));
 	ibuf_fd_set(b, pi[0]);
-	io_close_buffer(&rrdpq, b);
+	io_close_buffer(rrdpq, b);
 
 	http_fetch(id, uri, last_mod, pi[1]);
 }
@@ -393,7 +392,7 @@ rrdp_http_done(unsigned int id, enum http_result res, const char *last_mod)
 	io_simple_buffer(b, &id, sizeof(id));
 	io_simple_buffer(b, &res, sizeof(res));
 	io_str_buffer(b, last_mod);
-	io_close_buffer(&rrdpq, b);
+	io_close_buffer(rrdpq, b);
 }
 
 /*
@@ -460,7 +459,7 @@ queue_add_from_tal(struct tal *tal)
 	unsigned char	*data;
 	char		*nfile;
 
-	assert(tal->urisz);
+	assert(tal->num_uris > 0);
 
 	if ((taldescs[tal->id] = strdup(tal->descr)) == NULL)
 		err(1, NULL);
@@ -500,7 +499,7 @@ queue_add_from_cert(const struct cert *cert)
 
 	if (strncmp(cert->repo, RSYNC_PROTO, RSYNC_PROTO_LEN) != 0)
 		errx(1, "unexpected protocol");
-	host = cert->repo + 8;
+	host = cert->repo + RSYNC_PROTO_LEN;
 
 	LIST_FOREACH(le, &skiplist, entry) {
 		if (strncasecmp(host, le->fqdn, strcspn(host, "/")) == 0) {
@@ -639,6 +638,8 @@ entity_process(struct ibuf *b, struct stats *st, struct vrp_tree *tree,
 			break;
 		}
 		mft = mft_read(b);
+		if (mft->seqnum_gap)
+			repo_stat_inc(rp, talid, type, STYPE_SEQNUM_GAP);
 		queue_add_from_mft(mft);
 		mft_free(mft);
 		break;
@@ -764,6 +765,7 @@ sum_stats(const struct repo *rp, const struct repotalstats *in, void *arg)
 
 	out->mfts += in->mfts;
 	out->mfts_fail += in->mfts_fail;
+	out->mfts_gap += in->mfts_gap;
 	out->certs += in->certs;
 	out->certs_fail += in->certs_fail;
 	out->roas += in->roas;
@@ -878,6 +880,8 @@ load_skiplist(const char *slf)
 		LIST_INSERT_HEAD(&skiplist, le, entry);
 		stats.skiplistentries++;
 	}
+	if (ferror(fp))
+		err(1, "error reading %s", slf);
 
 	fclose(fp);
 	free(line);
@@ -968,12 +972,12 @@ suicide(int sig __attribute__((unused)))
 int
 main(int argc, char *argv[])
 {
-	int		 rc, c, i, st, proc, rsync, http, rrdp, hangup = 0;
+	int		 rc, c, i, st, hangup = 0;
+	int		 procfd, rsyncfd, httpfd, rrdpfd;
 	pid_t		 pid, procpid, rsyncpid, httppid, rrdppid;
 	struct pollfd	 pfd[NPFD];
 	struct msgbuf	*queues[NPFD];
-	struct ibuf	*b, *httpbuf = NULL, *procbuf = NULL;
-	struct ibuf	*rrdpbuf = NULL, *rsyncbuf = NULL;
+	struct ibuf	*b;
 	char		*rsync_prog = RPKI_RSYNC_CMD;
 	char		*bind_addr = NULL;
 	const char	*cachedir = NULL, *outputdir = NULL;
@@ -1009,8 +1013,12 @@ main(int argc, char *argv[])
 	    "proc exec unveil", NULL) == -1)
 		err(1, "pledge");
 
-	while ((c = getopt(argc, argv, "Ab:Bcd:e:fH:jmnoP:Rs:S:t:T:vVx")) != -1)
+	while ((c =
+	    getopt(argc, argv, "0Ab:Bcd:e:fH:jmnoP:Rs:S:t:vVx")) != -1)
 		switch (c) {
+		case '0':
+			excludeas0 = 0;
+			break;
 		case 'A':
 			excludeaspa = 1;
 			break;
@@ -1075,9 +1083,6 @@ main(int argc, char *argv[])
 				err(1, "too many tal files specified");
 			tals[talsz++] = optarg;
 			break;
-		case 'T':
-			bird_tablename = optarg;
-			break;
 		case 'v':
 			verbose++;
 			break;
@@ -1144,12 +1149,12 @@ main(int argc, char *argv[])
 	 * manifests, certificates, etc.) and returning contents.
 	 */
 
-	procpid = process_start("parser", &proc);
+	procpid = process_start("parser", &procfd);
 	if (procpid == 0) {
 		if (!filemode)
-			proc_parser(proc);
+			proc_parser(procfd);
 		else
-			proc_filemode(proc);
+			proc_filemode(procfd);
 	}
 
 	/* Constraints are only needed in the filemode and parser processes. */
@@ -1163,13 +1168,13 @@ main(int argc, char *argv[])
 	 */
 
 	if (!noop) {
-		rsyncpid = process_start("rsync", &rsync);
+		rsyncpid = process_start("rsync", &rsyncfd);
 		if (rsyncpid == 0) {
-			close(proc);
-			proc_rsync(rsync_prog, bind_addr, rsync);
+			close(procfd);
+			proc_rsync(rsync_prog, bind_addr, rsyncfd);
 		}
 	} else {
-		rsync = -1;
+		rsyncfd = -1;
 		rsyncpid = -1;
 	}
 
@@ -1180,15 +1185,15 @@ main(int argc, char *argv[])
 	 */
 
 	if (!noop && rrdpon) {
-		httppid = process_start("http", &http);
+		httppid = process_start("http", &httpfd);
 
 		if (httppid == 0) {
-			close(proc);
-			close(rsync);
-			proc_http(bind_addr, http);
+			close(procfd);
+			close(rsyncfd);
+			proc_http(bind_addr, httpfd);
 		}
 	} else {
-		http = -1;
+		httpfd = -1;
 		httppid = -1;
 	}
 
@@ -1199,15 +1204,15 @@ main(int argc, char *argv[])
 	 */
 
 	if (!noop && rrdpon) {
-		rrdppid = process_start("rrdp", &rrdp);
+		rrdppid = process_start("rrdp", &rrdpfd);
 		if (rrdppid == 0) {
-			close(proc);
-			close(rsync);
-			close(http);
-			proc_rrdp(rrdp);
+			close(procfd);
+			close(rsyncfd);
+			close(httpfd);
+			proc_rrdp(rrdpfd);
 		}
 	} else {
-		rrdp = -1;
+		rrdpfd = -1;
 		rrdppid = -1;
 	}
 
@@ -1227,14 +1232,18 @@ main(int argc, char *argv[])
 	if (pledge("stdio rpath wpath cpath fattr sendfd unveil", NULL) == -1)
 		err(1, "pledge");
 
-	msgbuf_init(&procq);
-	msgbuf_init(&rsyncq);
-	msgbuf_init(&httpq);
-	msgbuf_init(&rrdpq);
-	procq.fd = proc;
-	rsyncq.fd = rsync;
-	httpq.fd = http;
-	rrdpq.fd = rrdp;
+	if ((procq = msgbuf_new_reader(sizeof(size_t), io_parse_hdr, NULL)) ==
+	    NULL)
+		err(1, NULL);
+	if ((rsyncq = msgbuf_new_reader(sizeof(size_t), io_parse_hdr, NULL)) ==
+	    NULL)
+		err(1, NULL);
+	if ((httpq = msgbuf_new_reader(sizeof(size_t), io_parse_hdr, NULL)) ==
+	    NULL)
+		err(1, NULL);
+	if ((rrdpq = msgbuf_new_reader(sizeof(size_t), io_parse_hdr, NULL)) ==
+	   NULL)
+		err(1, NULL);
 
 	/*
 	 * The main process drives the top-down scan to leaf ROAs using
@@ -1242,14 +1251,14 @@ main(int argc, char *argv[])
 	 * parsing process.
 	 */
 
-	pfd[0].fd = proc;
-	queues[0] = &procq;
-	pfd[1].fd = rsync;
-	queues[1] = &rsyncq;
-	pfd[2].fd = http;
-	queues[2] = &httpq;
-	pfd[3].fd = rrdp;
-	queues[3] = &rrdpq;
+	pfd[0].fd = procfd;
+	queues[0] = procq;
+	pfd[1].fd = rsyncfd;
+	queues[1] = rsyncq;
+	pfd[2].fd = httpfd;
+	queues[2] = httpq;
+	pfd[3].fd = rrdpfd;
+	queues[3] = rrdpq;
 
 	load_skiplist(skiplistfile);
 
@@ -1306,16 +1315,13 @@ main(int argc, char *argv[])
 			if (pfd[i].revents & POLLHUP)
 				hangup = 1;
 			if (pfd[i].revents & POLLOUT) {
-				switch (msgbuf_write(queues[i])) {
-				case 0:
-					warnx("write[%d]: "
-					    "connection closed", i);
+				if (msgbuf_write(pfd[i].fd, queues[i]) == -1) {
+					if (errno == EPIPE)
+						warnx("write[%d]: "
+						    "connection closed", i);
+					else
+						warn("write[%d]", i);
 					hangup = 1;
-					break;
-				case -1:
-					warn("write[%d]", i);
-					hangup = 1;
-					break;
 				}
 			}
 		}
@@ -1330,8 +1336,13 @@ main(int argc, char *argv[])
 		 */
 
 		if ((pfd[1].revents & POLLIN)) {
-			b = io_buf_read(rsync, &rsyncbuf);
-			if (b != NULL) {
+			switch (ibuf_read(pfd[1].fd, queues[1])) {
+			case -1:
+				err(1, "ibuf_read");
+			case 0:
+				errx(1, "ibuf_read: connection closed");
+			}
+			while ((b = io_buf_get(queues[1])) != NULL) {
 				unsigned int id;
 				int ok;
 
@@ -1343,8 +1354,13 @@ main(int argc, char *argv[])
 		}
 
 		if ((pfd[2].revents & POLLIN)) {
-			b = io_buf_read(http, &httpbuf);
-			if (b != NULL) {
+			switch (ibuf_read(pfd[2].fd, queues[2])) {
+			case -1:
+				err(1, "ibuf_read");
+			case 0:
+				errx(1, "ibuf_read: connection closed");
+			}
+			while ((b = io_buf_get(queues[2])) != NULL) {
 				unsigned int id;
 				enum http_result res;
 				char *last_mod;
@@ -1362,8 +1378,14 @@ main(int argc, char *argv[])
 		 * Handle RRDP requests here.
 		 */
 		if ((pfd[3].revents & POLLIN)) {
-			b = io_buf_read(rrdp, &rrdpbuf);
-			if (b != NULL) {
+			switch (ibuf_read(pfd[3].fd, queues[3])) {
+			case -1:
+				abort();
+				err(1, "ibuf_read");
+			case 0:
+				errx(1, "ibuf_read: connection closed");
+			}
+			while ((b = io_buf_get(queues[3])) != NULL) {
 				rrdp_process(b);
 				ibuf_free(b);
 			}
@@ -1375,8 +1397,13 @@ main(int argc, char *argv[])
 		 */
 
 		if ((pfd[0].revents & POLLIN)) {
-			b = io_buf_read(proc, &procbuf);
-			if (b != NULL) {
+			switch (ibuf_read(pfd[0].fd, queues[0])) {
+			case -1:
+				err(1, "ibuf_read");
+			case 0:
+				errx(1, "ibuf_read: connection closed");
+			}
+			while ((b = io_buf_get(queues[0])) != NULL) {
 				entity_process(b, &stats, &vrps, &brks, &vaps,
 				    &vsps);
 				ibuf_free(b);
@@ -1397,10 +1424,10 @@ main(int argc, char *argv[])
 	 * This will cause them to exit, then we reap them.
 	 */
 
-	close(proc);
-	close(rsync);
-	close(http);
-	close(rrdp);
+	close(procfd);
+	close(rsyncfd);
+	close(httpfd);
+	close(rrdpfd);
 
 	rc = 0;
 	for (;;) {
@@ -1499,8 +1526,9 @@ main(int argc, char *argv[])
 	    stats.repo_tal_stats.certs, stats.repo_tal_stats.certs_fail);
 	printf("Trust Anchor Locators: %u (%u invalid)\n",
 	    stats.tals, talsz - stats.tals);
-	printf("Manifests: %u (%u failed parse)\n",
-	    stats.repo_tal_stats.mfts, stats.repo_tal_stats.mfts_fail);
+	printf("Manifests: %u (%u failed parse, %u seqnum gaps)\n",
+	    stats.repo_tal_stats.mfts, stats.repo_tal_stats.mfts_fail,
+	    stats.repo_tal_stats.mfts_gap);
 	printf("Certificate revocation lists: %u\n", stats.repo_tal_stats.crls);
 	printf("Ghostbuster records: %u\n", stats.repo_tal_stats.gbrs);
 	printf("Trust Anchor Keys: %u\n", stats.repo_tal_stats.taks);
@@ -1526,11 +1554,11 @@ main(int argc, char *argv[])
 
 usage:
 	fprintf(stderr,
-	    "usage: rpki-client [-ABcjmnoRVvx] [-b sourceaddr] [-d cachedir]"
+	    "usage: rpki-client [-0ABcjmnoRVvx] [-b sourceaddr] [-d cachedir]"
 	    " [-e rsync_prog]\n"
 	    "                   [-H fqdn] [-P epoch] [-S skiplist] [-s timeout]"
-	    " [-T table]\n"
-	    "                   [-t tal] [outputdir]\n"
+	    " [-t tal]\n"
+	    "                   [outputdir]\n"
 	    "       rpki-client [-Vv] [-d cachedir] [-j] [-t tal] -f file ..."
 	    "\n");
 	return 1;
