@@ -1,4 +1,4 @@
-/*	$OpenBSD: output-ometric.c,v 1.10 2024/04/08 14:02:13 tb Exp $ */
+/*	$OpenBSD: output-ometric.c,v 1.12 2024/11/02 12:30:28 job Exp $ */
 /*
  * Copyright (c) 2022 Claudio Jeker <claudio@openbsd.org>
  *
@@ -47,6 +47,8 @@ set_common_stats(const struct repotalstats *in, struct ometric *metric,
 	    OKV("type", "state"), OKV("manifest", "valid"), ol);
 	ometric_set_int_with_labels(metric, in->mfts_fail,
 	    OKV("type", "state"), OKV("manifest", "failed parse"), ol);
+	ometric_set_int_with_labels(metric, in->mfts_gap,
+	    OKV("type", "state"), OKV("manifest", "sequence gap"), ol);
 
 	ometric_set_int_with_labels(metric, in->roas,
 	    OKV("type", "state"), OKV("roa", "valid"), ol);
@@ -85,12 +87,14 @@ set_common_stats(const struct repotalstats *in, struct ometric *metric,
 	ometric_set_int_with_labels(metric, in->vaps_overflowed,
 	    OKV("type", "state"), OKV("vap overflowed"), ol);
 
-	ometric_set_int_with_labels(metric, in->spls,
-	    OKV("type", "state"), OKV("spl", "valid"), ol);
-	ometric_set_int_with_labels(metric, in->spls_fail,
-	    OKV("type", "state"), OKV("spl", "failed parse"), ol);
-	ometric_set_int_with_labels(metric, in->spls_invalid,
-	    OKV("type", "state"), OKV("spl", "invalid"), ol);
+	if (experimental) {
+		ometric_set_int_with_labels(metric, in->spls,
+		    OKV("type", "state"), OKV("spl", "valid"), ol);
+		ometric_set_int_with_labels(metric, in->spls_fail,
+		    OKV("type", "state"), OKV("spl", "failed parse"), ol);
+		ometric_set_int_with_labels(metric, in->spls_invalid,
+		    OKV("type", "state"), OKV("spl", "invalid"), ol);
+	}
 
 	ometric_set_int_with_labels(metric, in->vsps,
 	    OKV("type", "state"), OKV("vsp", "total"), ol);
