@@ -1,4 +1,4 @@
-/*	$OpenBSD: print.c,v 1.56 2024/09/12 10:33:25 tb Exp $ */
+/*	$OpenBSD: print.c,v 1.58 2024/11/13 12:51:04 tb Exp $ */
 /*
  * Copyright (c) 2021 Claudio Jeker <claudio@openbsd.org>
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
@@ -118,14 +118,14 @@ tal_print(const struct tal *p)
 		json_do_string("name", p->descr);
 		json_do_string("ski", pretty_key_id(ski));
 		json_do_array("trust_anchor_locations");
-		for (i = 0; i < p->urisz; i++)
+		for (i = 0; i < p->num_uris; i++)
 			json_do_string("tal", p->uri[i]);
 		json_do_end();
 	} else {
 		printf("Trust anchor name:        %s\n", p->descr);
 		printf("Subject key identifier:   %s\n", pretty_key_id(ski));
 		printf("Trust anchor locations:   ");
-		for (i = 0; i < p->urisz; i++) {
+		for (i = 0; i < p->num_uris; i++) {
 			if (i > 0)
 				printf("%26s", "");
 			printf("%s\n", p->uri[i]);
@@ -177,21 +177,21 @@ x509_print(const X509 *x)
 }
 
 static void
-as_resources_print(struct cert_as *as, size_t asz)
+as_resources_print(struct cert_as *ases, size_t num_ases)
 {
 	size_t i;
 
-	for (i = 0; i < asz; i++) {
+	for (i = 0; i < num_ases; i++) {
 		if (outformats & FORMAT_JSON)
 			json_do_object("resource", 1);
-		switch (as[i].type) {
+		switch (ases[i].type) {
 		case CERT_AS_ID:
 			if (outformats & FORMAT_JSON) {
-				json_do_uint("asid", as[i].id);
+				json_do_uint("asid", ases[i].id);
 			} else {
 				if (i > 0)
 					printf("%26s", "");
-				printf("AS: %u", as[i].id);
+				printf("AS: %u", ases[i].id);
 			}
 			break;
 		case CERT_AS_INHERIT:
@@ -206,14 +206,14 @@ as_resources_print(struct cert_as *as, size_t asz)
 		case CERT_AS_RANGE:
 			if (outformats & FORMAT_JSON) {
 				json_do_object("asrange", 1);
-				json_do_uint("min", as[i].range.min);
-				json_do_uint("max", as[i].range.max);
+				json_do_uint("min", ases[i].range.min);
+				json_do_uint("max", ases[i].range.max);
 				json_do_end();
 			} else {
 				if (i > 0)
 					printf("%26s", "");
-				printf("AS: %u -- %u", as[i].range.min,
-				    as[i].range.max);
+				printf("AS: %u -- %u", ases[i].range.min,
+				    ases[i].range.max);
 			}
 			break;
 		}
@@ -225,13 +225,13 @@ as_resources_print(struct cert_as *as, size_t asz)
 }
 
 static void
-ip_resources_print(struct cert_ip *ips, size_t ipsz, size_t asz)
+ip_resources_print(struct cert_ip *ips, size_t num_ips, size_t num_ases)
 {
 	char buf1[64], buf2[64];
 	size_t i;
 	int sockt;
 
-	for (i = 0; i < ipsz; i++) {
+	for (i = 0; i < num_ips; i++) {
 		if (outformats & FORMAT_JSON)
 			json_do_object("resource", 1);
 		switch (ips[i].type) {
@@ -239,7 +239,7 @@ ip_resources_print(struct cert_ip *ips, size_t ipsz, size_t asz)
 			if (outformats & FORMAT_JSON) {
 				json_do_bool("ip_inherit", 1);
 			} else {
-				if (i > 0 || asz > 0)
+				if (i > 0 || num_ases > 0)
 					printf("%26s", "");
 				printf("IP: inherit");
 			}
@@ -250,7 +250,7 @@ ip_resources_print(struct cert_ip *ips, size_t ipsz, size_t asz)
 			if (outformats & FORMAT_JSON) {
 				json_do_string("ip_prefix", buf1);
 			} else {
-				if (i > 0 || asz > 0)
+				if (i > 0 || num_ases > 0)
 					printf("%26s", "");
 				printf("IP: %s", buf1);
 			}
@@ -266,7 +266,7 @@ ip_resources_print(struct cert_ip *ips, size_t ipsz, size_t asz)
 				json_do_string("max", buf2);
 				json_do_end();
 			} else {
-				if (i > 0 || asz > 0)
+				if (i > 0 || num_ases > 0)
 					printf("%26s", "");
 				printf("IP: %s -- %s", buf1, buf2);
 			}
@@ -336,8 +336,8 @@ cert_print(const struct cert *p)
 		printf("Subordinate resources:    ");
 	}
 
-	as_resources_print(p->as, p->asz);
-	ip_resources_print(p->ips, p->ipsz, p->asz);
+	as_resources_print(p->ases, p->num_ases);
+	ip_resources_print(p->ips, p->num_ips, p->num_ases);
 
 	if (outformats & FORMAT_JSON)
 		json_do_end();
@@ -543,9 +543,8 @@ roa_print(const X509 *x, const struct roa *p)
 
 	if (outformats & FORMAT_JSON)
 		json_do_array("vrps");
-	for (i = 0; i < p->ipsz; i++) {
-		ip_addr_print(&p->ips[i].addr,
-		    p->ips[i].afi, buf, sizeof(buf));
+	for (i = 0; i < p->num_ips; i++) {
+		ip_addr_print(&p->ips[i].addr, p->ips[i].afi, buf, sizeof(buf));
 
 		if (outformats & FORMAT_JSON) {
 			json_do_object("vrp", 1);
@@ -601,8 +600,8 @@ spl_print(const X509 *x, const struct spl *s)
 
 	if (outformats & FORMAT_JSON)
 		json_do_array("prefixes");
-	for (i = 0; i < s->pfxsz; i++) {
-		ip_addr_print(&s->pfxs[i].prefix, s->pfxs[i].afi, buf,
+	for (i = 0; i < s->num_prefixes; i++) {
+		ip_addr_print(&s->prefixes[i].prefix, s->prefixes[i].afi, buf,
 		    sizeof(buf));
 
 		if (outformats & FORMAT_JSON) {
@@ -683,8 +682,8 @@ rsc_print(const X509 *x, const struct rsc *p)
 		printf("Signed with resources:    ");
 	}
 
-	as_resources_print(p->as, p->asz);
-	ip_resources_print(p->ips, p->ipsz, p->asz);
+	as_resources_print(p->ases, p->num_ases);
+	ip_resources_print(p->ips, p->num_ips, p->num_ases);
 
 	if (outformats & FORMAT_JSON) {
 		json_do_end();
@@ -692,7 +691,7 @@ rsc_print(const X509 *x, const struct rsc *p)
 	} else
 		printf("Filenames and hashes:     ");
 
-	for (i = 0; i < p->filesz; i++) {
+	for (i = 0; i < p->num_files; i++) {
 		if (base64_encode(p->files[i].hash, sizeof(p->files[i].hash),
 		    &hash) == -1)
 			errx(1, "base64_encode failure");
@@ -755,7 +754,7 @@ aspa_print(const X509 *x, const struct aspa *p)
 		printf("Providers:                ");
 	}
 
-	for (i = 0; i < p->providersz; i++) {
+	for (i = 0; i < p->num_providers; i++) {
 		if (outformats & FORMAT_JSON)
 			json_do_uint("asid", p->providers[i]);
 		else {
@@ -782,11 +781,11 @@ takey_print(char *name, const struct takey *t)
 		json_do_object("takey", 0);
 		json_do_string("name", name);
 		json_do_array("comments");
-		for (i = 0; i < t->commentsz; i++)
+		for (i = 0; i < t->num_comments; i++)
 			json_do_string("comment", t->comments[i]);
 		json_do_end();
 		json_do_array("uris");
-		for (i = 0; i < t->urisz; i++)
+		for (i = 0; i < t->num_uris; i++)
 			json_do_string("uri", t->uris[i]);
 		json_do_end();
 		json_do_string("spki", spki);
@@ -794,11 +793,11 @@ takey_print(char *name, const struct takey *t)
 	} else {
 		printf("TAL derived from the '%s' Trust Anchor Key:\n\n", name);
 
-		for (i = 0; i < t->commentsz; i++)
+		for (i = 0; i < t->num_comments; i++)
 			printf("\t# %s\n", t->comments[i]);
-		if (t->commentsz > 0)
+		if (t->num_comments > 0)
 			printf("\n");
-		for (i = 0; i < t->urisz; i++)
+		for (i = 0; i < t->num_uris; i++)
 			printf("\t%s\n", t->uris[i]);
 		printf("\n\t");
 		for (i = 0; i < strlen(spki); i++) {
@@ -886,7 +885,7 @@ geofeed_print(const X509 *x, const struct geofeed *p)
 		printf("Geofeed CSV records:      ");
 	}
 
-	for (i = 0; i < p->geoipsz; i++) {
+	for (i = 0; i < p->num_geoips; i++) {
 		if (p->geoips[i].ip->type != CERT_IP_ADDR)
 			continue;
 
