@@ -1,4 +1,4 @@
-/*	$OpenBSD: rsync.c,v 1.51 2024/08/20 13:31:49 claudio Exp $ */
+/*	$OpenBSD: rsync.c,v 1.56 2024/11/21 13:32:27 claudio Exp $ */
 /*
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
  *
@@ -71,7 +71,7 @@ rsync_base_uri(const char *uri)
 	}
 
 	/* Parse the non-zero-length hostname. */
-	host = uri + 8;
+	host = uri + RSYNC_PROTO_LEN;
 
 	if ((module = strchr(host, '/')) == NULL) {
 		warnx("%s: missing rsync module", uri);
@@ -226,17 +226,18 @@ proc_rsync(char *prog, char *bind_addr, int fd)
 {
 	int			 nprocs = 0, npending = 0, rc = 0;
 	struct pollfd		 pfd;
-	struct msgbuf		 msgq;
-	struct ibuf		*b, *inbuf = NULL;
+	struct msgbuf		*msgq;
+	struct ibuf		*b;
 	sigset_t		 mask, oldmask;
 	struct rsync		*s, *ns;
 
 	if (pledge("stdio rpath proc exec unveil", NULL) == -1)
 		err(1, "pledge");
 
+	if ((msgq = msgbuf_new_reader(sizeof(size_t), io_parse_hdr, NULL)) ==
+	    NULL)
+		err(1, NULL);
 	pfd.fd = fd;
-	msgbuf_init(&msgq);
-	msgq.fd = fd;
 
 	/*
 	 * Unveil the command we want to run.
@@ -294,7 +295,7 @@ proc_rsync(char *prog, char *bind_addr, int fd)
 
 		pfd.events = 0;
 		pfd.events |= POLLIN;
-		if (msgbuf_queuelen(&msgq) > 0)
+		if (msgbuf_queuelen(msgq) > 0)
 			pfd.events |= POLLOUT;
 
 		if (npending > 0 && nprocs < MAX_RSYNC_REQUESTS) {
@@ -343,7 +344,7 @@ proc_rsync(char *prog, char *bind_addr, int fd)
 				b = io_new_buffer();
 				io_simple_buffer(b, &s->id, sizeof(s->id));
 				io_simple_buffer(b, &ok, sizeof(ok));
-				io_close_buffer(&msgq, b);
+				io_close_buffer(msgq, b);
 
 				rsync_free(s);
 				nprocs--;
@@ -355,11 +356,11 @@ proc_rsync(char *prog, char *bind_addr, int fd)
 		}
 
 		if (pfd.revents & POLLOUT) {
-			switch (msgbuf_write(&msgq)) {
-			case 0:
-				errx(1, "write: connection closed");
-			case -1:
-				err(1, "write");
+			if (msgbuf_write(fd, msgq) == -1) {
+				if (errno == EPIPE)
+					errx(1, "write: connection closed");
+				else
+					err(1, "write");
 			}
 		}
 
@@ -370,30 +371,35 @@ proc_rsync(char *prog, char *bind_addr, int fd)
 		if (!(pfd.revents & POLLIN))
 			continue;
 
-		b = io_buf_read(fd, &inbuf);
-		if (b == NULL)
-			continue;
+		switch (ibuf_read(fd, msgq)) {
+		case -1:
+			err(1, "ibuf_read");
+		case 0:
+			errx(1, "ibuf_read: connection closed");
+		}
 
-		/* Read host and module. */
-		io_read_buf(b, &id, sizeof(id));
-		io_read_str(b, &dst);
-		io_read_str(b, &compdst);
-		io_read_str(b, &uri);
+		while ((b = io_buf_get(msgq)) != NULL) {
+			/* Read host and module. */
+			io_read_buf(b, &id, sizeof(id));
+			io_read_str(b, &dst);
+			io_read_str(b, &compdst);
+			io_read_str(b, &uri);
 
-		ibuf_free(b);
+			ibuf_free(b);
 
-		if (dst != NULL) {
-			rsync_new(id, uri, dst, compdst);
-			npending++;
-		} else {
-			TAILQ_FOREACH(s, &states, entry)
-				if (s->id == id)
-					break;
-			if (s != NULL) {
-				if (s->pid != 0)
-					kill(s->pid, SIGTERM);
-				else
-					rsync_free(s);
+			if (dst != NULL) {
+				rsync_new(id, uri, dst, compdst);
+				npending++;
+			} else {
+				TAILQ_FOREACH(s, &states, entry)
+					if (s->id == id)
+						break;
+				if (s != NULL) {
+					if (s->pid != 0)
+						kill(s->pid, SIGTERM);
+					else
+						rsync_free(s);
+				}
 			}
 		}
 	}
@@ -405,6 +411,6 @@ proc_rsync(char *prog, char *bind_addr, int fd)
 		rsync_free(s);
 	}
 
-	msgbuf_clear(&msgq);
+	msgbuf_free(msgq);
 	exit(rc);
 }
