@@ -1,4 +1,4 @@
-/*	$OpenBSD: validate.c,v 1.72 2024/02/22 12:49:42 job Exp $ */
+/*	$OpenBSD: validate.c,v 1.78 2024/11/12 09:23:07 tb Exp $ */
 /*
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
  *
@@ -43,14 +43,14 @@ valid_as(struct auth *a, uint32_t min, uint32_t max)
 		return 0;
 
 	/* Does this certificate cover our AS number? */
-	c = as_check_covered(min, max, a->cert->as, a->cert->asz);
+	c = as_check_covered(min, max, a->cert->ases, a->cert->num_ases);
 	if (c > 0)
 		return 1;
 	else if (c < 0)
 		return 0;
 
 	/* If it inherits, walk up the chain. */
-	return valid_as(a->parent, min, max);
+	return valid_as(a->issuer, min, max);
 }
 
 /*
@@ -69,60 +69,15 @@ valid_ip(struct auth *a, enum afi afi,
 		return 0;
 
 	/* Does this certificate cover our IP prefix? */
-	c = ip_addr_check_covered(afi, min, max, a->cert->ips, a->cert->ipsz);
+	c = ip_addr_check_covered(afi, min, max, a->cert->ips,
+	    a->cert->num_ips);
 	if (c > 0)
 		return 1;
 	else if (c < 0)
 		return 0;
 
 	/* If it inherits, walk up the chain. */
-	return valid_ip(a->parent, afi, min, max);
-}
-
-/*
- * Make sure the AKI is the same as the AKI listed on the Manifest,
- * and that the SKI doesn't already exist.
- * Return the parent by its AKI, or NULL on failure.
- */
-struct auth *
-valid_ski_aki(const char *fn, struct auth_tree *auths,
-    const char *ski, const char *aki, const char *mftaki)
-{
-	struct auth *a;
-
-	if (mftaki != NULL) {
-		if (strcmp(aki, mftaki) != 0) {
-			warnx("%s: AKI doesn't match Manifest AKI", fn);
-			return NULL;
-		}
-	}
-
-	if (auth_find(auths, ski) != NULL) {
-		warnx("%s: RFC 6487: duplicate SKI", fn);
-		return NULL;
-	}
-
-	a = auth_find(auths, aki);
-	if (a == NULL)
-		warnx("%s: RFC 6487: unknown AKI", fn);
-
-	return a;
-}
-
-/*
- * Validate a trust anchor by making sure that the SKI is unique.
- * Returns 1 if valid, 0 otherwise.
- */
-int
-valid_ta(const char *fn, struct auth_tree *auths, const struct cert *cert)
-{
-	/* SKI must not be a dupe. */
-	if (auth_find(auths, cert->ski) != NULL) {
-		warnx("%s: RFC 6487: duplicate SKI", fn);
-		return 0;
-	}
-
-	return 1;
+	return valid_ip(a->issuer, afi, min, max);
 }
 
 /*
@@ -136,26 +91,26 @@ valid_cert(const char *fn, struct auth *a, const struct cert *cert)
 	size_t		 i;
 	uint32_t	 min, max;
 
-	for (i = 0; i < cert->asz; i++) {
-		if (cert->as[i].type == CERT_AS_INHERIT)
+	for (i = 0; i < cert->num_ases; i++) {
+		if (cert->ases[i].type == CERT_AS_INHERIT)
 			continue;
 
-		if (cert->as[i].type == CERT_AS_ID) {
-			min = cert->as[i].id;
-			max = cert->as[i].id;
+		if (cert->ases[i].type == CERT_AS_ID) {
+			min = cert->ases[i].id;
+			max = cert->ases[i].id;
 		} else {
-			min = cert->as[i].range.min;
-			max = cert->as[i].range.max;
+			min = cert->ases[i].range.min;
+			max = cert->ases[i].range.max;
 		}
 
 		if (valid_as(a, min, max))
 			continue;
 
-		as_warn(fn, "RFC 6487: uncovered resource", &cert->as[i]);
+		as_warn(fn, "RFC 6487: uncovered resource", &cert->ases[i]);
 		return 0;
 	}
 
-	for (i = 0; i < cert->ipsz; i++) {
+	for (i = 0; i < cert->num_ips; i++) {
 		if (cert->ips[i].type == CERT_IP_INHERIT)
 			continue;
 
@@ -180,9 +135,9 @@ valid_roa(const char *fn, struct cert *cert, struct roa *roa)
 	size_t	 i;
 	char	 buf[64];
 
-	for (i = 0; i < roa->ipsz; i++) {
+	for (i = 0; i < roa->num_ips; i++) {
 		if (ip_addr_check_covered(roa->ips[i].afi, roa->ips[i].min,
-		    roa->ips[i].max, cert->ips, cert->ipsz) > 0)
+		    roa->ips[i].max, cert->ips, cert->num_ips) > 0)
 			continue;
 
 		ip_addr_print(&roa->ips[i].addr, roa->ips[i].afi, buf,
@@ -202,7 +157,8 @@ valid_roa(const char *fn, struct cert *cert, struct roa *roa)
 int
 valid_spl(const char *fn, struct cert *cert, struct spl *spl)
 {
-	if (as_check_covered(spl->asid, spl->asid, cert->as, cert->asz) > 0)
+	if (as_check_covered(spl->asid, spl->asid, cert->ases,
+	    cert->num_ases) > 0)
 		return 1;
 
 	warnx("%s: SPL: uncovered ASID: %u", fn, spl->asid);
@@ -350,6 +306,7 @@ build_chain(const struct auth *a, STACK_OF(X509) **intermediates,
 	*intermediates = NULL;
 	*root = NULL;
 
+	/* XXX - this should be removed, but filemode relies on it. */
 	if (a == NULL)
 		return;
 
@@ -357,7 +314,7 @@ build_chain(const struct auth *a, STACK_OF(X509) **intermediates,
 		err(1, "sk_X509_new_null");
 	if ((*root = sk_X509_new_null()) == NULL)
 		err(1, "sk_X509_new_null");
-	for (; a != NULL; a = a->parent) {
+	for (; a != NULL; a = a->issuer) {
 		assert(a->cert->x509 != NULL);
 		if (!a->any_inherits) {
 			if (!sk_X509_push(*root, a->cert->x509))
@@ -487,25 +444,25 @@ valid_rsc(const char *fn, struct cert *cert, struct rsc *rsc)
 	size_t		i;
 	uint32_t	min, max;
 
-	for (i = 0; i < rsc->asz; i++) {
-		if (rsc->as[i].type == CERT_AS_ID) {
-			min = rsc->as[i].id;
-			max = rsc->as[i].id;
+	for (i = 0; i < rsc->num_ases; i++) {
+		if (rsc->ases[i].type == CERT_AS_ID) {
+			min = rsc->ases[i].id;
+			max = rsc->ases[i].id;
 		} else {
-			min = rsc->as[i].range.min;
-			max = rsc->as[i].range.max;
+			min = rsc->ases[i].range.min;
+			max = rsc->ases[i].range.max;
 		}
 
-		if (as_check_covered(min, max, cert->as, cert->asz) > 0)
+		if (as_check_covered(min, max, cert->ases, cert->num_ases) > 0)
 			continue;
 
-		as_warn(fn, "RSC ResourceBlock uncovered", &rsc->as[i]);
+		as_warn(fn, "RSC ResourceBlock uncovered", &rsc->ases[i]);
 		return 0;
 	}
 
-	for (i = 0; i < rsc->ipsz; i++) {
+	for (i = 0; i < rsc->num_ips; i++) {
 		if (ip_addr_check_covered(rsc->ips[i].afi, rsc->ips[i].min,
-		    rsc->ips[i].max, cert->ips, cert->ipsz) > 0)
+		    rsc->ips[i].max, cert->ips, cert->num_ips) > 0)
 			continue;
 
 		ip_warn(fn, "RSC ResourceBlock uncovered", &rsc->ips[i]);
@@ -556,7 +513,7 @@ valid_aspa(const char *fn, struct cert *cert, struct aspa *aspa)
 {
 
 	if (as_check_covered(aspa->custasid, aspa->custasid,
-	    cert->as, cert->asz) > 0)
+	    cert->ases, cert->num_ases) > 0)
 		return 1;
 
 	warnx("%s: ASPA: uncovered Customer ASID: %u", fn, aspa->custasid);
@@ -574,10 +531,10 @@ valid_geofeed(const char *fn, struct cert *cert, struct geofeed *g)
 	size_t	 i;
 	char	 buf[64];
 
-	for (i = 0; i < g->geoipsz; i++) {
+	for (i = 0; i < g->num_geoips; i++) {
 		if (ip_addr_check_covered(g->geoips[i].ip->afi,
 		    g->geoips[i].ip->min, g->geoips[i].ip->max, cert->ips,
-		    cert->ipsz) > 0)
+		    cert->num_ips) > 0)
 			continue;
 
 		ip_addr_print(&g->geoips[i].ip->ip, g->geoips[i].ip->afi, buf,
@@ -632,7 +589,7 @@ valid_uuid(const char *s)
 static int
 valid_ca_pkey_rsa(const char *fn, EVP_PKEY *pkey)
 {
-	RSA		*rsa;
+	const RSA	*rsa;
 	const BIGNUM	*rsa_e;
 	int		 key_bits;
 
@@ -663,7 +620,7 @@ valid_ca_pkey_rsa(const char *fn, EVP_PKEY *pkey)
 static int
 valid_ca_pkey_ec(const char *fn, EVP_PKEY *pkey)
 {
-	EC_KEY		*ec;
+	const EC_KEY	*ec;
 	const EC_GROUP	*group;
 	int		 nid;
 	const char	*cname;
